#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle      hnd;
    int                    rc;
    int                    ret;
    int                    devIdx;
    char                  *devName = DEFAULT_DEV;
    char                   fullDevName[32];
    char                  *pgmname = argv[0];
    TSYNC_ERROR            err     = TSYNC_SUCCESS;
    unsigned int           index;
    TSYNC_TimeObj          doyTime;
    TSYNC_TimeBCDObj       bcdTime;
    unsigned int           seconds;
    unsigned int           ns;
    TSYNC_TimeDSTStateObj  ds;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: MR_GetTime <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    ret = snprintf(fullDevName, sizeof(fullDevName), "%s%s%d",
                   DEVICE_ROOT, devName, devIdx);

    if (ret < 0 || ret >= sizeof(fullDevName))
    {
        printf("Error formating device name\n");
        return (1);
    }

    ret = 0;

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    index = atoi(argv[2]);
    err = TSYNC_MR_getTime(hnd, index, &doyTime);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
        *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d %d %d %d %d",
                doyTime.years, doyTime.doy, doyTime.hours,
                doyTime.minutes, doyTime.seconds);
    }

    printf("\n");
    printf(" DOY Time:\n");
    printf("  Year: %04d\n", doyTime.years);
    printf("  DOY:  %03d\n", doyTime.doy);
    printf("  Hr:   %02d\n", doyTime.hours);
    printf("  Min:  %02d\n", doyTime.minutes);
    printf("  Sec:  %02d\n", doyTime.seconds);
    printf("  Nsec: %09d\n", doyTime.ns);

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
