#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    TSYNC_PTPMacAddrPwObj   macAddrPw;


    /* If invalid number of arguments... */
    if (argc != 5)
    {
        printf(" Usage: PTR_SetMacAddr <device index> <index>  <mac addr> <pw>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Ethernet ITF message */
    index = atoi(argv[2]);
    sscanf(argv[3],"%2hhx:%2hhx:%2hhx:%2hhx:%2hhx:%2hhx",
                   &(macAddrPw.macAddress[0]),     &(macAddrPw.macAddress[1]),
                   &(macAddrPw.macAddress[2]),     &(macAddrPw.macAddress[3]),
                   &(macAddrPw.macAddress[4]),     &(macAddrPw.macAddress[5]));
    sscanf(argv[4],"%12s", macAddrPw.pw);


    macAddrPw.macAddress[6] = 0;
    macAddrPw.macAddress[7] = 0;
                               
    printf("\n");
    printf(" PTR (%d) MAC Address information:\n", (index));
    printf("  MAC Address  : %02x.%02x.%02x.%02x.%02x.%02x\n", 
           macAddrPw.macAddress[0], macAddrPw.macAddress[1], macAddrPw.macAddress[2],
           macAddrPw.macAddress[3], macAddrPw.macAddress[4], macAddrPw.macAddress[5]);
    printf("  Password     : %s\n", macAddrPw.pw);

    
    
    // Send Set MAC Address Info transaction
    err = TSYNC_PTR_setMacAddr(hnd, index, &macAddrPw);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
