#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPPortStateObj    portStateObj;


    /* If invalid number of arguments... */
    if (argc != 11)
    {
        printf(" Usage: PTR_SetPortState <device index> <index> <Port Number> <Port Enable> <Port State> <Link Connected> <slave log Ann> <slave log Sync> <slave log Delay Req> <slave One Step Mode>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Port State message */
    index = atoi(argv[2]);
    portStateObj.portNumber       = atoi(argv[ 3]);
    portStateObj.portEnabled      = atoi(argv[ 4]);
    portStateObj.portState        = (PTL_PTP_STATE)atoi(argv[ 5]);
    portStateObj.linkConnected    = atoi(argv[ 6]);
    portStateObj.slaveLogAnn      = atoi(argv[ 7]);
    portStateObj.slaveLogSync     = atoi(argv[ 8]);
    portStateObj.slaveLogDelayReq = atoi(argv[ 9]);
    portStateObj.slaveOneStepMode = atoi(argv[10]);

    printf("\n");
    printf(" PTR (%d) Clock Settings information:\n", (index));
    printf("  Port Number   : %i\n", portStateObj.portNumber);
    printf("  Port Enabled  : %i\n", portStateObj.portEnabled);
    printf("  PTP State     : %i\n", portStateObj.portState);
    printf("  Link State    : %i\n", portStateObj.linkConnected);
    printf("  Slave Announce Interval (log)  : %i\n", portStateObj.slaveLogAnn);
    printf("  Slave Sync Interval (Log)      : %i\n", portStateObj.slaveLogSync);
    printf("  Slave Delay Req Interval (log) : %i\n", portStateObj.slaveLogDelayReq);
    printf("  Slave One Step Mode            : %i\n", portStateObj.slaveOneStepMode);
    
    // Send Set Port State transaction
    err = TSYNC_PTR_setPortState(hnd, index, &portStateObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
