#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle      hnd;
    int                    rc;
    int                    ret     = 0;
    int                    devIdx;
    unsigned int           index;
    char                  *devName = DEFAULT_DEV;
    char                   fullDevName[32];
    char                  *pgmname = argv[0];
    TSYNC_ERROR            err     = TSYNC_SUCCESS;
    TSYNC_StrSubObj        sub;
    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: STR_GetSubcription <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Position message */
    index = atoi(argv[2]);
    err = TSYNC_STR_getSubscription(hnd, index, &sub);
    
    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
    * On definition of the environment variable SPEC_KTSAL_EN
    * to true, print out the information in the format used
    * for the equivalent set's command line.
    */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
        *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%s %s %d %d", sub.beginSub, sub.endSub, sub.state, sub.unused);
    }

    printf("\n");
    printf(" Begin of Subscription: %s\n", sub.beginSub);
    printf(" End of Subscription: %s\n", sub.endSub);
    
    switch (sub.state)
    {
        case 0 : printf("\tSubscription OK\n"); break;
        case 1 : printf("\tSubscription within 90 days of expiration\n"); break;
        case 2 : printf("\tSubscription within 30 days of expiration\n"); break;
        case 3 : printf("\tSubscription within 7 days of expiration\n"); break;
        case 4 : printf("\tSubscription expired\n");break;
        default : printf("\tSubscription unknown\n");break;
    }
      
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
